;Filename: TVal.s
;Part: ADuC7020,ADuC7024,ADuC7026.
;Created: 01/09/2004
;Updated: 28/06/2005
;Author: Barry Broderick
;This file is in beta sampling.

#include        <ADuC702x.IAR.equ>

; TVal==========Writes/reads timer T value and starts/stops timer.
; C Function prototype: int TVal(int iT, int iTLd, int iTGo);
;iT:{0,1,2,3}
;iTLd:{0-0xffffffff} -> 32bit reload value for T1, 16 bit for all others
;iTGo:{T_STOP,T_RUN,T_RD}
; Description of Function: Sets the reload value of T and starts/stops T.
; User interface: 
;  Set iT to desired Timer, 0 to 3
;   Set iTGo to one of the following values:
;       0 = T_STOP -> Stop T.
;       1 = T_RUN  -> write TLD and Run T. 
;       2 = T_RD   -> only read TVAL. 
;   Call TVal().
;       Depending on iTGo Stop T or run T with 
;       iTLd as start value or only read T1VAL.
;   Returns current TVAL in  r0.
; Robustness:
;   Interupt functionality may be added separately.
; Side effects: Corrupts r1 and r3.

                RSEG CODE
                MODULE TVal
                PUBLIC TVal
                CODE32

TVal:    cmp     r2,#1                 ;check iTGo.
         mov     r3,#0xff0000          ;r3 = Z_BASE
         orr     r3,r3,#0xff000000
         orr     r3,r3,r0,lsl#5        ;r3 = r0 << 5;

         bhi     TRVal                 ;if(iTGo>1) only read TVAL.
         ldr     r2,[r3,#Z_T0CON]      ;r2 = TCON;
         andmi   r2,r2,#0x7f           ;if(iTGo<1) clear enable bit.
         orreq   r2,r2,#0x80           ;if(iTGo==1) set enable bit.
         str     r2,[r3,#Z_T0CON]      ;T0CON = r2;
         streq   r1,[r3,#Z_T0LD]       ;if(iTGo==1) TLD = iTLd;

TRVal:   ldr     r0,[r3,#Z_T0VAL]      ;return TVAL;
         bx      lr


                 ENDMOD  TVal

                 END
        
